#!/bin/ksh
#
# itmpac_config_tbsm.sh
#
# Description: Initialize TBSM for IBM Tivoli Monitoring ProActive Components.
#
#              Read file to determine steps to define objects, icon image,
#              task libraries and Lines of Business.
#
#              The directory that contains this script must also contain
#              all icon and tll files.
#
#              This script must be run on the TBSM server under the MKS
#              Korn shell.
#
# 5724-C01
# IBM Tivoli Monitoring
# (C) COPYRIGHT IBM Corp. 2004
# All Rights Reserved
# Licensed Material - Property of IBM Corp.
#
# 140047 - Add Automated Business Systems support
#

CONFIG_FILE="itmpacConfigTbsm.file"
WORKDIR="${TEMP}/itmpac"
LOG="${WORKDIR}/trace_itmpac_config_tbsm.log"

FIRSTABSCMD="true"

# Exit if non-zero return code
# TBSM scripts will explain the error

function checkrc {

  if [ $1 -ne 0 ]
  then

    exit $1

  fi
}

#
# If object does not already exist, create it with gemgenprod.sh command.
#

function createobj {

COMPANY=`echo $line | awk -F":" '{print $2}'`
SOFTCOMP=`echo $line | awk -F":" '{print $3}'`
VERSIONS=`echo $line | awk -F":" '{print $4}' | tr "," " "`

if [ "$COMPANY" = "" ] || [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ]; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  # Loop on version
  for VERSION in $VERSIONS
  do

    # If object already exists, do not try to create it
    set +x
    RESULT=`${TBSMBINDIR}/tbsmsql.exe -MCONSOLE -r -dObject $args -Q"exec gemsp_list_products" | grep "$SOFTCOMP $VERSION,"`
    rc=$?
    set -x
    if [ $rc -eq 0 ]; then
      echo "$SOFTCOMP $VERSION already exists."
      continue
    fi

    echo "${TBSMBINDIR}/gemgenprod.sh -m $COMPANY -p $SOFTCOMP -v $VERSION $log_args"
    set +x
    RESULT=`${TBSMBINDIR}/gemgenprod.sh -m "$COMPANY" -p "$SOFTCOMP" -v "$VERSION" $args`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi


}

#
# If TBSM 1.5, run gemimageimport.sh to import an icon for an object
#

function importgif {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
SIZE=`echo $line | awk -F":" '{print $4}'`
FILENAME=`echo $line | awk -F":" '{print $5}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$SIZE" = "" ] || [ "$FILENAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Run only if TBSM 1.5

  if [ "$isTBSM2" = "TRUE" ]; then

    echo "IMPORGIF: Ignore since this is not TBSM 1.5."
    return

  fi

  # Process action

  cp ${CURRDIR}/${FILENAME} ${WORKDIR}/${FILENAME}
  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    echo "${TBSMBINDIR}/gemimageimport.sh -p $SOFTCOMP -v $VERSION -${SIZE} -i $FILENAME $log_args"
    set +x
    RESULT=`${TBSMBINDIR}/gemimageimport.sh -p "$SOFTCOMP" -v "$VERSION" -${SIZE} -i $FILENAME $args`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi
}

#
# If TBSM 2.1, run loadgemicons.ksh to load an icon for an object
#

function loadgif {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
FILENAME=`echo $line | awk -F":" '{print $4}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$FILENAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  # Run only if TBSM 2.1 or greater

  if [ "$isTBSM2" = "FALSE" ]; then

    echo "LOADGIF: Ignore since this is TBSM 1.5."
    return

  fi

  cp ${CURRDIR}/${FILENAME} ${WORKDIR}/${FILENAME}
  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    echo "${TBSMBINDIR}/loadgemicons.ksh -p $SOFTCOMP -v $VERSION -f $FILENAME $log_args"
    set +x
    RESULT=`${TBSMBINDIR}/loadgemicons.ksh -p "$SOFTCOMP" -v "$VERSION" -f $FILENAME $args`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi

}

#
# Define tasks for an object using tllparser.sh and bldgemtasks.sh commands.
# Optionally, provide a timeout value for the task.
#

function tllparser {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
LIBNAME=`echo $line | awk -F":" '{print $4}'`
FILENAME=`echo $line | awk -F":" '{print $5}'`
TIMEOUT=`echo $line | awk -F":" '{print $6}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$LIBNAME" = "" ] || [ "$FILENAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cp ${CURRDIR}/${FILENAME} ${WORKDIR}/${FILENAME}
  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    # Convert tll files to sqi files

    ${TLLBINDIR}/tllparser.sh -p "$SOFTCOMP" -v "$VERSION" -o . -f $FILENAME
    checkrc $?

    # Modify timeout value if set

    if [ "$TIMEOUT" != "" ]; then

      mv ${FILENAME}.sqi ${FILENAME}.tmp
      cat ${FILENAME}.tmp | sed "s/cmd OUTPUT/cmd OUTPUT, $TIMEOUT/" >${FILENAME}.sqi

    fi

    # Build the tasks from the sqi files

    echo "${TBSMBINDIR}/bldgemtasks.sh -p $SOFTCOMP -v $VERSION -t $LIBNAME -f ${FILENAME}.sqi $log_args"
    set +x
    RESULT=`${TBSMBINDIR}/bldgemtasks.sh -p "$SOFTCOMP" -v "$VERSION" -t "$LIBNAME" -f ${FILENAME}.sqi $args`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi

}

#
# Define tasks for an object using only the bldgemtasks.sh command.
#

function bldgemtasks {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
LIBNAME=`echo $line | awk -F":" '{print $4}'`
FILENAME=`echo $line | awk -F":" '{print $5}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$LIBNAME" = "" ] || [ "$FILENAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cp ${CURRDIR}/${FILENAME} ${WORKDIR}/${FILENAME}
  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    # Build the tasks from the sqi files

    echo "${TBSMBINDIR}/bldgemtasks.sh -p $SOFTCOMP -v $VERSION -t $LIBNAME -f $FILENAME $log_args"
    set +x
    RESULT=`${TBSMBINDIR}/bldgemtasks.sh -p "$SOFTCOMP" -v "$VERSION" -t "$LIBNAME" -f $FILENAME $args`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi

}

#
# Add Line of Business definition using the addLOB.sh command.
#

function addlob {

LOBNAME=`echo $line | awk -F":" '{print $2}'`
PARENTLOBNAME=`echo $line | awk -F":" '{print $3}'`

if [ "$LOBNAME" = "" ] || [ "$PARENTLOBNAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  echo "${TBSMBINDIR}/addLOB.sh -l $LOBNAME -p $PARENTLOBNAME $log_args"
  set +x
  RESULT=`${TBSMBINDIR}/addLOB.sh -l "$LOBNAME" -p "$PARENTLOBNAME" $args`
  rc=$?
  echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
  set -x
  checkrc $rc

fi

}

#
# Define a line of business for a sofware component with the addpvToLob.sh 
# command.
#

function addpvtolob {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
LOBNAME=`echo $line | awk -F":" '{print $4}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$LOBNAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    # Add Software Component to the Line of Business

    echo "${TBSMBINDIR}/addpvToLOB.sh -p $SOFTCOMP -v $VERSION -l $LOBNAME $log_args"
    set +x
    RESULT=`${TBSMBINDIR}/addpvToLOB.sh -p "$SOFTCOMP" -v "$VERSION" -l "$LOBNAME" $args`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi

}

#
# Add software component information to a temp file for later processing
# by the function that creates an Automated Business System template file.
#

function addpvtoabs {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
LOBNAME=`echo $line | awk -F":" '{print $4}'`
PARENTLOB=`echo $line | awk -F":" '{print $5}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$LOBNAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    # Get the class of the object

    set +x
    RESULT=`${TBSMBINDIR}/tbsmsql.exe -MCONSOLE -r -dObject $args -Q"exec gemsp_list_products" | grep "$SOFTCOMP $VERSION,"`    
    rc=$?
    set -x
    if [ $rc -eq 0 ]; then
      class=`echo $RESULT | awk -F"," '{print $2}'`
      class=`echo $class`
    fi

    # Use the LOBNAME to create the pathname

    pathname=`echo "$LOBNAME" | sed -e "s/ /_/g"`

    # Use the pathname for the TMPFILE
    TMPFILE="$WORKDIR/${pathname}_ABS.file"

    # If this is the first abs command, delete any old temp files
    if [ "$FIRSTABSCMD" = "true" ]; then

      rm -fr $TMPFILE
      FIRSTABSCMD="false"

    fi

    saveline="$class:$pathname:$LOBNAME:$PARENTLOB"
    echo "$saveline" >>$TMPFILE

  done
fi

}

#
# Based on the temp file created by the addpvtoabs function, create
# a template file to be used by the TBSM Administrator for Automated
# Business Systems.
#

function createAbsFile {

  # Create an absfiles directory and put the abs template file there

  ABSDIR="${TBSMDATDIR}/absfiles"

  if [ ! -d $ABSDIR ]; then

    mkdir $ABSDIR
    checkrc $?

  fi

  TMPBASE=`basename $TMPFILE`
  ABSFILE="${ABSDIR}/${TMPBASE}"

  echo "PatternList" >$ABSFILE
  echo "ListName	Operand" >>$ABSFILE
  echo "" >>$ABSFILE
  echo "Pattern" >>$ABSFILE
  echo "Pattern	Class	Attribute	When	Operator	Operand1	Operand2" >>$ABSFILE

  i=1
  while read absline
  do

    class=`echo "$absline" | awk -F":" '{print $1}'`
    echo "$i	$class	name	Current	LIKE	%	%" >>$ABSFILE
    i=`expr $i + 1`

  done <$TMPFILE

  echo "" >>$ABSFILE
  echo "CriteriaToPattern" >>$ABSFILE
  echo "Criteria	Pattern	PatternRelated" >>$ABSFILE

  i=1
  while read absline
  do

    echo "100$i	$i	$i" >>$ABSFILE
    i=`expr $i + 1`

  done <$TMPFILE

  echo "" >>$ABSFILE
  echo "Path" >>$ABSFILE
  echo "Path	Level	Name" >>$ABSFILE

  pathname=`head -1 $TMPFILE | awk -F":" '{print $2}'`
  lobname=`head -1 $TMPFILE | awk -F":" '{print $3}'`
  parentlobname=`head -1 $TMPFILE | awk -F":" '{print $4}'`
  
  echo "$pathname	1	$parentlobname" >>$ABSFILE
  echo "$pathname	2	$lobname" >>$ABSFILE
  echo "$pathname	3" >>$ABSFILE

  echo "" >>$ABSFILE
  echo "CriteriaToPath" >>$ABSFILE
  echo "Criteria	Path	Level	Pattern	Variable	Value" >>$ABSFILE

  i=1
  while read absline
  do

    echo "100$i	$pathname	3	$i	name	<$i:name>" >>$ABSFILE
    i=`expr $i + 1`

  done <$TMPFILE
}

#
# Remove an object definition from the database
#

function removeobj {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ]; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  # Stop the application server service

  sc stop ASIApplicationSvc

  for VERSION in $VERSIONS
  do

    # Delete software components

    cmd="${TBSMBINDIR}/tbsmsql.exe -MCONSOLE -r -dObject $log_args -Qexec gemsp_delete_product @prodname='$SOFTCOMP', @version='$VERSION'"
    set +x
    RESULT=`${TBSMBINDIR}/tbsmsql.exe -MCONSOLE -r -dObject $args -Q"exec gemsp_delete_product @prodname='$SOFTCOMP', @version='$VERSION'"`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

  # Restart the application server service

  sc start ASIApplicationSvc
  sleep 10

fi

}

#
# Remove a line of business definition for a software component using the
# addpvToLOB.sh -a delete command.
#

function rempvtolob {

SOFTCOMP=`echo $line | awk -F":" '{print $2}'`
VERSIONS=`echo $line | awk -F":" '{print $3}' | tr "," " "`
LOBNAME=`echo $line | awk -F":" '{print $4}'`

if [ "$SOFTCOMP" = "" ] || [ "$VERSIONS" = "" ] || [ "$LOBNAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  for VERSION in $VERSIONS
  do

    # Remove a Software Component from the Line of Business

    echo "${TBSMBINDIR}/addpvToLOB.sh -p $SOFTCOMP -v $VERSION -l $LOBNAME $log_args -a delete"
    set +x
    RESULT=`${TBSMBINDIR}/addpvToLOB.sh -p "$SOFTCOMP" -v "$VERSION" -l "$LOBNAME" $args -a delete`
    rc=$?
    echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
    set -x
    checkrc $rc

  done

fi
}

#
# Remove a line of business definition using the rmLOB.sh and
# updateLOBview.sh commands.
#

function remlob {

LOBNAME=`echo $line | awk -F":" '{print $2}'`
PARENTLOBNAME=`echo $line | awk -F":" '{print $3}'`

if [ "$LOBNAME" = "" ] || [ "$PARENTLOBNAME" = "" ] ; then

  echo "Parameter missing from: $line"

else

  # Process action

  cd $WORKDIR

  # Ignore return codes because they may be misleading

  echo "${TBSMBINDIR}/rmLOB.sh -l $LOBNAME -p $PARENTLOBNAME $log_args"
  set +x
  RESULT=`${TBSMBINDIR}/rmLOB.sh -l "$LOBNAME" -p "$PARENTLOBNAME" $args`
  echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
  set -x

  echo "${TBSMBINDIR}/rmLOB.sh -l $PARENTLOBNAME $log_args"
  set +x
  RESULT=`${TBSMBINDIR}/rmLOB.sh -l "$PARENTLOBNAME" $args`
  echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
  set -x

  echo "${TBSMBINDIR}/updateLOBview.sh $log_args"
  set +x
  RESULT=`${TBSMBINDIR}/updateLOBview.sh $args`
  echo "$RESULT" | sed "s/-P $PASSWORD/-P xxxxxxxx/g"
  set -x

fi

}

#
# Main
#

CURRDIR=`pwd`

# Set directory names based on where TBSM is installed
# This command works for TBSM 1.5 and 2.1 

TBSMDIR=`regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //'`

if [ ! -d $TBSMDIR ]; then

  echo "Directory not found: $TBSMDIR"
  exit 1

fi

TBSMBINDIR="${TBSMDIR}/bin"
TLLBINDIR="${TBSMDIR}/xdfparser/bin"
TBSMDATDIR="${TBSMDIR}/Data"

PATH="$PATH;$TBSMBINDIR;$TLLBINDIR"
export PATH

# Determine TBSM version

if [ -x "${TBSMBINDIR}/loadgemicons.ksh" ]; then

  isTBSM2="TRUE"

else

  isTBSM2="FALSE"

fi

# Check Usage
# On failure, use a TBSM translated message but strip out the [].
# Our usage is:
#
# $0 -S server -U userid -P password

if [ "$1" != "-S" ] || [ "$3" != "-U" ] || [ "$5" != "-P" ] || [ "$6" = "" ]
then

  echo
  ${TBSMBINDIR}/ASIShellMsg ASILobScripts 15 $0 | sed 's/\[//g' | sed 's/\]//g'
  echo

  exit 1

fi

# Check for optional arguments for config file

if [ "$7" = "-F" ] && [ "$8" != "" ]; then

  CONFIG_FILE="$8"

fi

# We will pass the database command line arguments to commands that needs them

args="$1 $2 $3 $4 $5 $6"
PASSWORD=$6

# Do not put real password in the logs

log_args="$1 $2 $3 $4 $5 xxxxxxxx"

# Create a working directory and move our files there

if [ ! -d $WORKDIR ]; then

  mkdir $WORKDIR
  checkrc $?

fi

exec 2>$LOG
set -x

if [ ! -r ${CURRDIR}/${CONFIG_FILE} ]; then

  echo "Configuration file $CONFIG_FILE not found."
  exit 1

fi

while read line
do

ACTION=`echo $line | awk -F":" '{print $1}'`

case "$ACTION" in


  CREATEOBJ)	createobj $line;;

  IMPORTGIF)	importgif $line;;

  LOADGIF)	loadgif $line;;

  TLLPARSER)	tllparser $line;;

  BLDGEMTASKS)	bldgemtasks $line;;

  ADDLOB)	addlob $line;;

  ADDPVTOLOB)	addpvtolob $line;;

  ADDPVTOABS)	addpvtoabs $line;;

  REMOVEOBJ)	removeobj $line;;

  REMPVTOLOB)	rempvtolob $line;;

  REMLOB)	remlob $line;;

  *)		echo "Unknown action: $ACTION";;

esac


done < ${CURRDIR}/${CONFIG_FILE}

#
# If FIRSTABSCMD is false, then we have processed at least one ABS
# command and we should create an ABS file
#

if [ "$FIRSTABSCMD" = "false" ]; then

  createAbsFile

fi


exit 0
